#python

#---Import---
import lx #@UnresolvedImport
import os
import sys
import inspect
import re
import math
from fs_modoModules.runtime import RunTime
from fs_modoModules.dialogs import Dialogs
from fs_modoModules.items import Items
from fs_modoModules.utilities import Utilities
from fs_modoModules.channels import Channels

#---Info---
lx.out("--------------------------------------------------")
lx.out("Author: Fredrik Stenson // fredrik@stenson.tv")
lx.out("www.modo.stenson.tv")
lx.out("Script: fs_OctaneExporter.py")
lx.out("Version: 1.05 // Designed For: 401 and OctaneRender 1.022b beta2.2b")
lx.out("")

#---Create Class Instances--- 
runtime = RunTime()
dialog = Dialogs()
item = Items()
util = Utilities()
chan = Channels()

#---Functions---

def cam_matrix():
    function = sys._getframe().f_code.co_name
    try:
        target_world = [0, 0, 0]
        cam_world_pos = lx.eval('query sceneservice item.worldPos ? %s' %cam_id)
        target_distance = float(focusDist *-1)
        target_local = util.matrix_x_vector(lx.eval('query sceneservice item.worldMatrix ? %s' %cam_id), [0, 0, target_distance])
        target_world[0] = target_local[0] + cam_world_pos[0]
        target_world[1] = target_local[1] + cam_world_pos[1]
        target_world[2] = target_local[2] + cam_world_pos[2]
        
        up_local = util.matrix_x_vector(lx.eval('query sceneservice item.worldMatrix ? %s' %cam_id), [0, 1, 0])
        up_local = util.normalize_vector(up_local)
        
        return cam_world_pos, target_world, up_local
    
    except:
        runtime.error_function(function)
        lx.out('%s - Scripts Line' %(inspect.currentframe().f_back.f_lineno))
        lx.out()

##################################################################################

def get_fov(cam):
    function = sys._getframe().f_code.co_name
    try:
        lx.eval('select.typeFrom item')
        lx.eval('select.item %s' %cam)
        fov = lx.eval('camera.hfov ?')
        return math.degrees(fov)

    except:
        runtime.error_function(function)
        lx.out('%s - Scripts Line' %(inspect.currentframe().f_back.f_lineno))
        lx.out()

##################################################################################

def freeze_temp_scene():
    function = sys._getframe().f_code.co_name
    try:
        cur_frame = lx.eval('select.time ?') * lx.eval('time.fpsCustom ?')
        
        if walk == False:
            #load temp scene... (same as revert)
            lx.eval('!!scene.open "%s" normal' %temp_scene)
        
        lx.eval('select.time [%s f] [0] [0]' %cur_frame)
        lx.eval('item.refSystem {}')
        
        #Freeze the geo...
        lx.eval('select.drop item')
        item.select_all_item_type('mesh')
        lx.eval('select.typeFrom polygon')
        lx.eval('poly.freeze false false 2 false true')
        
        if walk == False or 'render_frame' in args:
            lx.eval('copy')
            lx.eval('scene.new')
            lx.eval('paste')
            
        if prj_unit == 'mm':
            prj_scale = 0.001
        if prj_unit == 'cm':
            prj_scale = 0.01
        if prj_unit == 'dm':
            prj_scale = 0.1
        if prj_unit == 'm':
            prj_scale = 1.0
        if prj_unit == 'dcm':
            prj_scale = 10.0
        if prj_unit == 'hcm':
            prj_scale = 100.0
        if prj_unit == 'km':
            prj_scale = 1000.0
        if prj_unit == 'inch':
            prj_scale = 0.0254
        if prj_unit == 'feet':
            prj_scale = 0.3048
        if prj_unit == 'yard':
            prj_scale = 0.9144
        if prj_unit == 'fur':
            prj_scale = 201.168
        if prj_unit == 'mile':
            prj_scale = 1609.344
            
        scene_sclae = prj_scale
        
        lx.eval('tool.set TransformScale on')
        lx.eval('tool.attr xfrm.transform SX %s' %scene_sclae)
        lx.eval('tool.attr xfrm.transform SY %s' %scene_sclae)
        lx.eval('tool.attr xfrm.transform SZ %s' %scene_sclae)
        lx.eval('tool.doApply')
        lx.eval('tool.set TransformScale off 0')
        
        lx.eval('!!scene.saveAs "%s" wf_OBJ true' %obj_export_path)
        
        if walk == False or 'render_frame' in args:
            lx.eval('!!scene.close')
            lx.eval('scene.set %s' %temp_scene_id)

        
    except:
        runtime.error_function(function)
        lx.out('%s - Scripts Line' %(inspect.currentframe().f_back.f_lineno))
        lx.out()

##################################################################################

def export_imgSequence(export_frame):
    function = sys._getframe().f_code.co_name
    try:
        frame_pad = util.create_padding(export_frame, 4)
        
        img_paths = util.get_file_from_path(img_path_ext, '.png')
        img_name_ext = img_paths[0] + frame_pad + '.png'
        
        return img_paths[1] + img_name_ext
    
    except:
        runtime.error_function(function)
        lx.out('%s - Scripts Line' %(inspect.currentframe().f_back.f_lineno))
        lx.out()

##################################################################################
   
def export_gpuProps():
    function = sys._getframe().f_code.co_name
    try:
        gpu_use = ''
        if gpu_act == True:
            for gpu in gpu_nums.split(' '):
                gpu_use+=' -g '+gpu
            return gpu_use
        else:
            return ''
        
    except:
        runtime.error_function(function)
        lx.out('%s - Scripts Line' %(inspect.currentframe().f_back.f_lineno))
        lx.out()

##################################################################################

def export_filmProps():
    function = sys._getframe().f_code.co_name
    try:
        if res_act == True:
            return '--film-width %s --film-height %s' %(int(res_width * res_percent), int(res_height * res_percent))
        else:
            return ''
        
    except:
        runtime.error_function(function)
        lx.out('%s - Scripts Line' %(inspect.currentframe().f_back.f_lineno))
        lx.out()

##################################################################################

def export_sunProps():
    function = sys._getframe().f_code.co_name
    try:
        if light_act == True:
            sun_id = lx.eval('environment.light sunLight:?')
            target_local = util.matrix_x_vector(lx.eval('query sceneservice item.worldMatrix ? %s' %sun_id), [0, 0, 1])
            return ' --daylight-sundir-x ' + str(target_local[0]) + ' --daylight-sundir-y ' + str(target_local[1]) + ' --daylight-sundir-z ' + str(target_local[2])
        else:
            return ''
        
    except:
        runtime.error_function(function)
        lx.out('%s - Scripts Line' %(inspect.currentframe().f_back.f_lineno))
        lx.out()

##################################################################################

def export_cameraProps():
    function = sys._getframe().f_code.co_name
    try:
        cameraProps = ''
        if cam_act == True:
            
            cam_world_pos = cam_matrix()[0]
            target_world = cam_matrix()[1]
            up_local = cam_matrix()[2]
            
            fov = get_fov(cam_id)
            
            cameraProps += ' --cam-pos-x ' + str(cam_world_pos[0]) + ' --cam-pos-y ' + str(cam_world_pos[1]) + ' --cam-pos-z ' + str(cam_world_pos[2]) + ' --cam-target-x ' + str(target_world[0]) + ' --cam-target-y ' + str(target_world[1]) + ' --cam-target-z ' + str(target_world[2]) + ' --cam-up-x ' + str(up_local[0]) + ' --cam-up-y ' + str(up_local[1]) + ' --cam-up-z ' + str(up_local[2])
            cameraProps += ' --cam-fov ' + str(fov)
            
            if aperture_act == True:
                if aperture_fetch == True:
                    fStop = float(chan.q_channel_value(cam_id, 'fStop'))
                    diameter = (focalLen/fStop) * 100
                    radius = diameter / 2
                    cameraProps += ' --cam-aperture ' + str(radius)
                else:
                    cameraProps += ' --cam-aperture ' + str(aperture_radius*100)
                
            if focaldepth_act == True:
                if focaldepth_fetch == True:
                    cameraProps += ' --cam-focaldepth ' + str(focusDist)
                else:
                    cameraProps += ' --cam-focaldepth ' + str(focal_depth)
                
            if motionblur_act == True:
                if motionblur_order == 'previous':
                    cur_frame_motion = cur_frame + 1
                if motionblur_order == 'next':
                    cur_frame_motion = cur_frame - 1
                    
                lx.eval('select.time [%s f] [0] [0]' %cur_frame_motion)
                
                cam_world_pos = cam_matrix()[0]
                target_world = cam_matrix()[1]
                up_local = cam_matrix()[2]
                
                cameraProps += ' --cam-motion-pos-x ' + str(cam_world_pos[0]) + ' --cam-motion-pos-y ' + str(cam_world_pos[1]) + ' --cam-motion-pos-z ' + str(cam_world_pos[2]) + ' --cam-motion-target-x ' + str(target_world[0]) + ' --cam-motion-target-y ' + str(target_world[1]) + ' --cam-motion-target-z ' + str(target_world[2]) + ' --cam-motion-up-x ' + str(up_local[0]) + ' --cam-motion-up-y ' + str(up_local[1]) + ' --cam-motion-up-z ' + str(up_local[2])
                
                lx.eval('select.time [%s f] [0] [0]' %cur_frame)
        
        return cameraProps
    
    except:
        runtime.error_function(function)
        lx.out('%s - Scripts Line' %(inspect.currentframe().f_back.f_lineno))
        lx.out()

##################################################################################

def create_path(path, name, extention):
    function = sys._getframe().f_code.co_name
    try:
        if sys.platform == 'darwin':
            path_count = len(path) -1
            if path[path_count] == '/':
                full_path = path + name + extention
            else:
                full_path = path + '/' + name + extention
            lx.out('PATH: ' + full_path)
            return full_path
        elif sys.platform == 'linux':
            pass
        else:
            path_count = len(path) -1
            if path[path_count] == '\\':
                full_path = path + name + extention
            else:   
                full_path = path + '\\' + name + extention
                
            lx.out('PATH: ' + full_path)
            return full_path
        
    except:
        runtime.error_function(function)
        lx.out('%s - Scripts Line' %(inspect.currentframe().f_back.f_lineno))
        lx.out()

##################################################################################

#---Variables---
args = lx.args()

scene_name = lx.eval('query sceneservice scene.name ? selected')
scene_path = lx.eval('query sceneservice scene.file ? selected')
scene_n = lx.eval('query sceneservice scene.N ?')
scene_id = lx.eval('scene.set ?')

cur_frame = lx.eval('select.time ?') * lx.eval('time.fpsCustom ?')

app_path = lx.eval('user.value fs_octane_app_path ?')
prj_path = lx.eval('user.value fs_octane_prj_path ?')
prj_name = lx.eval('user.value fs_octane_prj_name ?')
prj_unit = lx.eval('user.value fs_octane_units ?')

res_act = lx.eval('user.value fs_octane_res_act ?')
res_width = lx.eval('user.value fs_octane_width_res ?')
res_height = lx.eval('user.value fs_octane_height_res ?')
res_percent = lx.eval('user.value fs_octane_percent_res ?')

cam_act = lx.eval('user.value fs_octane_cam_act ?')
cam_id = lx.eval('render.camera ?')

aperture_act = lx.eval('user.value fs_octane_aperture_act ?')
aperture_radius = lx.eval('user.value fs_octane_aperture ?')
aperture_fetch = lx.eval('user.value fs_octane_aperture_fetch ?')

focaldepth_act = lx.eval('user.value fs_octane_focal_act ?')
focal_depth = lx.eval('user.value fs_octane_focal ?')
focaldepth_fetch = lx.eval('user.value fs_octane_focal_fetch ?')
focalLen = float(chan.q_channel_value(cam_id, 'focalLen'))
focusDist = float(chan.q_channel_value(cam_id, 'focusDist'))

motionblur_act = lx.eval('user.value fs_octane_motionblur_act ?')
motionblur_order = lx.eval('user.value fs_octane_motion ?')

light_act = lx.eval('user.value fs_octane_light_act ?')

gpu_act = lx.eval('user.value fs_octane_gpu_act ?')
gpu_nums = lx.eval('user.value fs_octane_gpu_nums ?')

img_path_ext = lx.eval('user.value fs_octane_img_path ?')

frame_start = lx.eval('user.value fs_octane_start_frame ?')
frame_end = lx.eval('user.value fs_octane_end_frame ?')
frame_step = lx.eval('user.value fs_octane_step_frame ?')

pixel_samples = lx.eval('user.value fs_octane_samples ?')

walk = lx.eval('user.value fs_octane_walk_act ?')
lx.out('Walk Thru Mode: %s '  %walk)

app_path = '"' + app_path + '"'

try:
    #Create temp scene to freeze mesh in...
    if 'render_frame' in args or 'render_animation' in args:
        temp_scene = create_path(prj_path, 'freeze_temp_scene', '.lxo')
        lx.eval('!!scene.saveAs "%s" $LXOB true' %temp_scene)
        lx.eval('!!scene.open "%s" normal' %temp_scene)
        temp_scene_id = lx.eval('scene.set ?')

#---Project Configuration---#############################################################
    if 'app_path' in args:
        app_path = dialog.open_costume_file_dialog('Octane Binary', 'exe', 'exe', '*.exe', '.exe')
        lx.eval('user.value fs_octane_app_path "%s"' %app_path)
        lx.out('App Path: %s' %app_path)
        
    if 'prj_path' in args:
        prj_path = dialog.gett_path_dialog('Project Path')
        lx.eval('user.value fs_octane_prj_path "%s"' %prj_path)
        lx.out('Prj Path: %s' %prj_path)

    if 'prj_name' in args:
        prj_path_ext = dialog.open_costume_file_dialog('Browse ocs file', 'ocs', 'ocs', '*.ocs', '.ocs')
        
        prj_paths = util.get_file_from_path(prj_path_ext, '.ocs')
        
        lx.eval('user.value fs_octane_prj_path "%s"' %prj_paths[1])
        lx.eval('user.value fs_octane_prj_name "%s"' %prj_paths[0])
        lx.out('Use Existing: %s' %prj_paths[1])

#---EXPORT---#############################################################
    if 'img_path' in args:
        img_path_ext = dialog.save_costume_file_dialog('Save Sequense', 'png', 'PNG', '*.png', 'png')[0]
        lx.eval('user.value fs_octane_img_path "%s"' %img_path_ext)

#---Spaces Check---
    if ' ' in img_path_ext:
        dialog.error_dialog('Space in path', 'There is a space in the images sequence path! Remove all spaces.')
        sys.exit('Space in images sequence path!')
    if ' ' in prj_path:
        dialog.error_dialog('Space in path', 'There is a space in the project path! Remove all spaces.')
        sys.exit('Space in project path!')
    if ' ' in prj_name:
        dialog.error_dialog('Space in path', 'There is a space in the project name! Remove all spaces.')
        sys.exit('Space in project name!')
    
#---Export OBJ Only---
    if 'export_obj' in args:
        obj_export_path = create_path(prj_path, prj_name, '.obj')
        lx.eval('!scene.saveAs "%s" wf_OBJ true' %obj_export_path)
        
        dialog.info_dialog('Exported', 'The file has been exported to: %s' %obj_export_path)
        
#---Render Still---           
    if 'render_frame' in args:
        
        ocs_export_path = create_path(prj_path, prj_name, '.ocs')
        obj_export_path = create_path(prj_path, prj_name, '.obj')
        
        freeze_temp_scene()
        
        camera = export_cameraProps()
        sun = export_sunProps()
        film = export_filmProps()
        gpu = export_gpuProps()

#---Update OCS---        
        if os.path.exists(ocs_export_path):
           
            shell_command = app_path + ' -m ' + prj_name + ' -r ' + obj_export_path + ' ' + ocs_export_path + ' ' + camera + ' ' + sun + ' ' + film + ' ' + gpu
            lx.out('RUN: ' + shell_command)
            os.system(shell_command + ' &')
            
            dialog.info_dialog('Rendering...', 'The project has been exported to: %s' %prj_path)

#---Create OCS---       
        else:
            
            shell_command = app_path + ' -n ' + ocs_export_path + ' -m ' + prj_name + ' -l ' + obj_export_path + ' ' + camera + ' ' + sun + ' ' + film + ' ' + gpu
            lx.out('RUN: ' + shell_command)
            os.system(shell_command + ' &')
            
            dialog.info_dialog('Rendering...', 'The project has been exported to: %s' %prj_path)

#---Render Animation---                
    if 'render_animation' in args:

        monitor = lx.Monitor()
        monitor.init(frame_end-frame_start)
        
        ocs_export_path = create_path(prj_path, prj_name, '.ocs')
        obj_export_path = create_path(prj_path, prj_name, '.obj')
        
        
        for i in range(frame_start, frame_end+1, frame_step):

            lx.eval('select.time [%s f] [0] [0]' %i) 
            
            freeze_temp_scene()
            
            camera = export_cameraProps()
            sun = export_sunProps()
            film = export_filmProps()
            gpu = export_gpuProps()
            img = export_imgSequence(i)

#---Update OCS--- 
            if os.path.exists(ocs_export_path):
                shell_command = app_path + ' -e -q -o ' + img + ' -s ' + str(pixel_samples) + ' -m ' + prj_name + ' -r ' + obj_export_path + ' ' + camera + ' ' + sun + ' ' + film + ' ' + gpu + ' ' + ocs_export_path
                lx.out('RUN: ' + shell_command)
                os.system(shell_command)
#---Create OCS---                
            else:
                dialog.error_dialog('Create OCS', 'You need to choose an existing project for animation rendering!')
                sys.exit('No OCS for animation rendering found!')

            monitor.step(frame_step)
    
    #close the temp scene...
    if 'render_frame' in args or 'render_animation' in args:
        for i in range(int(lx.eval('query sceneservice scene.N ?'))):
            j = i+1
            lx.eval('scene.set %s' %j)
            if temp_scene_id == lx.eval('scene.set ?'):
                lx.eval('!!scene.close')
                lx.eval('scene.set %s' %scene_id)
                break
        
except:
    runtime.error_script()
    
lx.out("--------------------------------------------------")